using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Win32;
using System.Drawing;

namespace FileTypeAndIcon
{
    public static class RegisteredFileType
    {
        #region APIs

        [DllImport("shell32.dll", EntryPoint = "ExtractIconA", CharSet = CharSet.Ansi, SetLastError = true, ExactSpelling = true)]
        public static extern IntPtr ExtractIcon(int hInst, string lpszExeFileName, int nIconIndex);

        [DllImport("Shell32.dll")]
        public extern static int ExtractIconEx(string libName, int iconIndex,IntPtr[] largeIcon, IntPtr[] smallIcon, int nIcons);

        [StructLayout(LayoutKind.Sequential)]
        internal struct ShellFileInfo
        {
            public IntPtr handle;
            public int iIcon;
            public uint attributes;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 260)]
            public char[] displayName;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 80)]
            public char[] typeName;
        }

        [DllImport("shell32")]
        internal static extern IntPtr SHGetFileInfo(string path, uint fileAttributes, ref ShellFileInfo fileInfo, uint cbFileInfo, uint flags);


        #endregion

        private static Hashtable filesAndIcons;

        static RegisteredFileType()
        {
            filesAndIcons = GetFileTypeAndIcon();
        }

        public static Icon[] GetIconFromExtension(string fileType)
        {
            try
            {
                string fileAndParam = (filesAndIcons[fileType.ToUpper()]).ToString();
                if (String.IsNullOrEmpty(fileAndParam))
                    return null;

                //Use to store the file contains icon.
                string fileName = "";

                //The index of the icon in the file.
                int iconIndex = 0;
                string iconIndexString = "";

                int index = fileAndParam.IndexOf(",");
                //if fileAndParam is some thing likes that: "C:\\Program Files\\NetMeeting\\conf.exe,1".
                if (index > 0)
                {
                    fileName = fileAndParam.Substring(0, index);
                    iconIndexString = fileAndParam.Substring(index + 1);
                }
                else
                    fileName = fileAndParam;

                if (!string.IsNullOrEmpty(iconIndexString))
                {
                    //Get the index of icon.
                    iconIndex = int.Parse(iconIndexString);

                    if (iconIndex == -1)
                        iconIndex = 0;
                }

                //Gets the handle of the icon.
                IntPtr[] largeIcon = new IntPtr[1];
                IntPtr[] smallIcon = new IntPtr[1];

                int result = RegisteredFileType.ExtractIconEx(fileName, iconIndex, largeIcon, smallIcon, 1);

                //The handle cannot be zero.
                Icon[] icons = new Icon[2];

                if (largeIcon[0] != IntPtr.Zero)
                {
                    //Gets the real icon.
                    icons[0] = Icon.FromHandle(largeIcon[0]);

                    if (icons[0].Height == 0 || icons[0].Width == 0)
                    {
                        icons[0] = null;
                    }
                }

                if (smallIcon[0] != IntPtr.Zero)
                {
                    //Gets the real icon.
                    icons[1] = Icon.FromHandle(smallIcon[0]);

                    if (icons[1].Height == 0 || icons[1].Width == 0)
                    {
                        icons[1] = null;
                    }
                }

                return icons;

            }
            catch
            {
                return null;
            }
        }

        public static Icon[] GetShellIconsFromExtension(string fileType)
        {
            try
            {
                Icon[] icons = new Icon[2];
                uint fileAttributes = 0x80;
                uint flags = 0x100;

                // For extention we do flags |= 0x10
                flags |= 0x10;

                ShellFileInfo fileInfo = new ShellFileInfo();
                uint shfiSize = (uint)Marshal.SizeOf(fileInfo.GetType());

                IntPtr ret = SHGetFileInfo(
                    fileType, fileAttributes, ref fileInfo, shfiSize, flags);

                if (ret != IntPtr.Zero)
                    if (fileInfo.handle != IntPtr.Zero)
                        icons[0] = System.Drawing.Icon.FromHandle(fileInfo.handle);

                // For small Icon
                flags |= 1;
                fileInfo = new ShellFileInfo();
                shfiSize = (uint)Marshal.SizeOf(fileInfo.GetType());

                ret = SHGetFileInfo(
                    fileType, fileAttributes, ref fileInfo, shfiSize, flags);

                if (ret != IntPtr.Zero)
                    if (fileInfo.handle != IntPtr.Zero)
                        icons[1] = System.Drawing.Icon.FromHandle(fileInfo.handle);

                return icons;

            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// Gets registered file types and their associated icon in the system.
        /// </summary>
        /// <returns>Returns a hash table which contains the file extension as keys, the icon file and param as values.</returns>
        public static Hashtable GetFileTypeAndIcon()
        {
            try
            {
                // Create a registry key object to represent the HKEY_CLASSES_ROOT registry section
                RegistryKey rkRoot = Registry.ClassesRoot;

                //Gets all sub keys' names.
                string[] keyNames = rkRoot.GetSubKeyNames();
                Hashtable iconsInfo = new Hashtable();

                //Find the file icon.
                foreach (string keyName in keyNames)
                {
                    if (String.IsNullOrEmpty(keyName))
                        continue;
                    int indexOfPoint = keyName.IndexOf(".");
                    
                    //If this key is not a file exttension(eg, .zip), skip it.
                    if (indexOfPoint != 0)
                        continue;

                    RegistryKey rkFileType = rkRoot.OpenSubKey(keyName);
                    if (rkFileType == null)
                        continue;

                    //Gets the default value of this key that contains the information of file type.
                    object defaultValue = rkFileType.GetValue("");
                    if (defaultValue == null)
                        continue;

                    //Go to the key that specifies the default icon associates with this file type.
                    string defaultIcon = defaultValue.ToString() + "\\DefaultIcon";
                    RegistryKey rkFileIcon = rkRoot.OpenSubKey(defaultIcon);
                    if (rkFileIcon != null)
                    {
                        //Get the file contains the icon and the index of the icon in that file.
                        object value = rkFileIcon.GetValue("");
                        if (value != null)
                        {
                            //Clear all unecessary " sign in the string to avoid error.
                            string fileParam = value.ToString().Replace("\"", "");
                            iconsInfo.Add(keyName.ToUpper(), fileParam);
                        }
                        rkFileIcon.Close();
                    }
                    rkFileType.Close();
                }
                rkRoot.Close();
                return iconsInfo;
            }
            catch (Exception exc)
            {
                throw exc;
            }
        }
    }
}
